import { defineComponent, onMounted, reactive, ref, nextTick } from 'vue';
import styles from './index.module.less';
import {
  Cell,
  CellGroup,
  Col,
  List,
  Picker,
  Popup,
  Row,
  Tab,
  Tabs,
  DatePicker,
  Button
} from 'vant';
import MSticky from '@/components/m-sticky';
import personIcon from './images/personIcon.png';
import homeIcon from './images/homeIcon.png';
import memberIcon from './images/memberIcon.png';
import sanIcon from './images/san.png';
import iconQrcode from './images/icon-qrcode.png';
import qrcodeBg from './images/qrcode-bg.png';
import qrbg from './images/qr-bg.png';
import request from '@/helpers/request';
import topDot from './images/topDot.png';
import { useRoute, useRouter } from 'vue-router';
// import { moneyFormat, numberFormat } from '@/helpers/utils';
import OFullRefresh from '@/components/m-full-refresh';
import OEmpty from '@/components/m-empty';
// import arrowIcon from './images/arrowIcon.png';
// import OHeader from '@/components/m-header';
import OSearch from '@/components/m-search';
import numeral from 'numeral';
import MQrcode from '@/components/m-qrcode';
import html2canvas from 'html2canvas';
export default defineComponent({
  name: 'tenant-apply-data',
  setup() {
    const route = useRoute();
    const forms = reactive({
      keyword: '',
      id: route.query.id,
      // id: '1687275949971763202',
      yearStatus: false,
      schoolId: null,
      year: [new Date().getFullYear()] as any,
      yearName: new Date().getFullYear(),
      classList: [] as any,
      statObj: {
        registerNum: 0,
        schoolNum: 0,
        registerMemberShipNum: 0
      } as any,
      perponStatus: false,
      sortKey: 'CLASS' as 'CLASS' | 'MEMBER',
      sortId: 'desc',
      sortName: '报名人数降序',
      sortList: [
        { value: 'desc', text: '报名人数降序' },
        { value: 'asc', text: '报名人数升序' },
        { value: 'mdesc', text: '会员人数降序' },
        { value: 'masc', text: '会员人数升序' }
      ] as any,
      page: 1,
      rows: 20,
      isClick: false,
      qrcodeStatus: false,
      url: '1', // 二维码
      urlItem: {} as any
    });

    const refreshing = ref(false);
    const loading = ref(true);
    const finished = ref(false);
    const showContact = ref(false);
    const list = ref([]);

    const getSchoolList = async () => {
      try {
        const { data } = await request.get('/edu-app/open/school/list', {
          params: {
            tenantId: forms.id
          }
        });
        const temp = [
          {
            value: '',
            text: '全部学校'
          }
        ];
        if (Array.isArray(data)) {
          data.forEach((item: any) => {
            temp.push({
              value: item.id,
              text: item.name
            });
          });
          forms.classList = temp;
        }
      } catch {
        //
      }
    };

    const getList = async () => {
      if (forms.isClick) {
        return;
      }
      forms.isClick = true;
      if (refreshing.value) {
        list.value = [];
        forms.page = 1;
        refreshing.value = false;
      }
      try {
        const res = await request.post(
          '/edu-app/open/school/schoolRegisterPage',
          {
            data: {
              tenantId: forms.id,
              page: forms.page,
              rows: forms.rows,
              year: forms.yearName,
              keyword: forms.keyword,
              sort: forms.sortId,
              sortKey: forms.sortKey
            }
          }
        );

        if (list.value.length > 0 && res.data.current === 1) {
          return;
        }

        list.value = list.value.concat(res.data.rows || []);
        forms.page = res.data.current + 1;
        showContact.value = list.value.length > 0;
        loading.value = false;
        finished.value = res.data.current >= res.data.pages;
      } catch {
        showContact.value = false;
        finished.value = true;
      } finally {
        loading.value = false;
      }
      forms.isClick = false;
    };
    const onRefresh = () => {
      finished.value = false;
      // 重新加载数据
      // 将 loading 设置为 true，表示处于加载状态
      loading.value = true;
      getList();
    };

    const imgs = reactive({
      saveLoading: false,
      image: null as any,
      shareLoading: false
    });
    const downImg = () => {
      if (imgs.saveLoading) {
        return;
      }
      imgs.saveLoading = true;
      // 判断是否已经生成图片
      // if (imgs.image) {
      //   saveImg();
      // } else {
      const container: any = document.getElementById(`preview-container`);
      console.log('1212121');
      html2canvas(container, {
        allowTaint: true,
        useCORS: true,
        backgroundColor: null
      })
        .then(async canvas => {
          const url = canvas.toDataURL('image/png');
          imgs.image = url;
          // saveImg();
          forms.qrcodeStatus = true;
          imgs.saveLoading = false;
        })
        .catch(() => {
          imgs.saveLoading = false;
          console.log('222');
        });
      // }
    };

    const saveImg = async () => {
      // showLoadingToast({ message: '图片生成中...', forbidClick: true });
      setTimeout(() => {
        imgs.saveLoading = false;
      }, 100);
      const link = document.createElement('a');
      link.setAttribute('download', forms.urlItem.schoolName + '统计' + '.png');
      // 添加时间戳，防止浏览器缓存图

      link.href = imgs.image;
      link.click();
    };

    onMounted(async () => {
      if (route.query.name) {
        document.title = route.query.name + '报名统计';
      } else {
        document.title = '学生报名统计';
      }

      // await getSchoolList();
      await getStat();
      await getList();
    });
    const getStat = async () => {
      try {
        const { data } = await request.post(
          '/edu-app/open/school/schoolRegisterStat',
          {
            data: {
              tenantId: forms.id,
              year: forms.yearName
            }
          }
        );
        forms.statObj = data;
      } catch {
        //
      }
    };
    return () => (
      <div class={styles.tenantAllData}>
        {/* <OHeader isBack={false} /> */}
        <MSticky position="top">
          <div class={styles.top}>
            <div class={styles.topWrap}>
              <div class={styles.topHead}>
                <img src={topDot} class={styles.topDot} alt="" />
                汇总数据
              </div>
              {/* <div
                class={styles.timerWrap}
                onClick={() => {
                  forms.yearStatus = true;
                }}>
                {forms.yearName}年{' '}
                <img
                  src={sanIcon}
                  class={[
                    styles.sanIcon,
                    forms.yearStatus ? styles.routeSan : ''
                  ]}
                  alt=""
                />
              </div> */}
            </div>
            <div class={styles.cardWrap}>
              <div class={[styles.schoolCard, styles.cardItem]}>
                <div class={styles.cardNum}>
                  {numeral(forms.statObj.schoolNum).format('0,0')}
                </div>
                <div class={styles.cardInfo}>
                  <img src={homeIcon} class={styles.cardInfoImg} alt="" />
                  学校数量
                </div>
                <div class={styles.cardLine}></div>
              </div>
              <div class={[styles.studentCard, styles.cardItem]}>
                <div class={styles.cardNum}>
                  {numeral(forms.statObj.registerNum).format('0,0')}
                </div>
                <div class={styles.cardInfo}>
                  <img src={personIcon} class={styles.cardInfoImg} alt="" />
                  报名人数
                </div>
                <div class={styles.cardLine}></div>
              </div>
              <div class={[styles.memberCard, styles.cardItem]}>
                <div class={styles.cardNum}>
                  {numeral(forms.statObj.registerMemberShipNum).format('0,0')}
                </div>
                <div class={styles.cardInfo}>
                  <img src={memberIcon} class={styles.cardInfoImg} alt="" />
                  会员人数
                </div>
                <div class={styles.cardLine}></div>
              </div>
            </div>
            <div class={styles.searchWrap}>
              <div class={styles.searechInfo}>
                <OSearch
                  class={styles.allDataWrap}
                  shape="round"
                  background="#F6F8F9"
                  inputBackground="white"
                  placeholder="请输入学校名称"
                  onSearch={val => {
                    forms.keyword = val;
                    forms.page = 1;
                    refreshing.value = true;
                    getList();
                  }}></OSearch>
              </div>

              <div
                class={styles.timerWrap}
                onClick={() => (forms.perponStatus = true)}>
                {forms.sortName}{' '}
                <img
                  src={sanIcon}
                  class={[
                    styles.sanIcon,
                    forms.perponStatus ? styles.routeSan : ''
                  ]}
                  alt=""
                />
              </div>
            </div>
          </div>
        </MSticky>

        <div class={styles.schoolList}>
          {showContact.value ? (
            <OFullRefresh
              v-model:modelValue={refreshing.value}
              onRefresh={onRefresh}
              class={styles.refreshC}>
              <List
                loading-text=" "
                finished={finished.value}
                finished-text=" "
                onLoad={getList}>
                {list.value.map((item: any) => (
                  <div class={styles.schoolItem}>
                    <div class={styles.schoolNameWrap}>
                      {/* <p class={styles.title}>学校名称</p> */}
                      <p class={styles.schoolName}>{item.schoolName}</p>
                    </div>
                    <div class={styles.schoolCountWrap}>
                      <div>
                        <p class={styles.personNum}>{item.registerNum || 0}</p>
                        <p class={styles.title}>报名人数</p>
                      </div>
                      <div>
                        <p class={styles.personNum}>
                          {item.registerMemberShipNum || 0}
                        </p>
                        <p class={styles.title}>会员人数</p>
                      </div>
                      <div
                        onClick={() => {
                          forms.urlItem = item;
                          forms.url = `${location.origin}/classroom-app/#/tenantDataShool?id=${item.schoolId}&name=${item.schoolName}`;

                          nextTick(() => {
                            downImg();
                          });
                        }}>
                        <p class={styles.personNum}>
                          <img src={iconQrcode} />
                        </p>
                        <p class={styles.title}>学校统计二维码</p>
                      </div>
                      {/* <img class={[styles.arrow]} src={arrowIcon} alt="" /> */}
                    </div>
                  </div>
                ))}
              </List>
            </OFullRefresh>
          ) : null}

          {!showContact.value && !loading.value && (
            <OEmpty description="暂无学校信息" class={styles.emptyC} />
          )}
        </div>
        <Popup
          v-model:show={forms.yearStatus}
          position="bottom"
          round
          class={'popupBottomSearch'}>
          <DatePicker
            showToolbar
            v-model={forms.year}
            columns-type={['year']}
            onCancel={() => (forms.yearStatus = false)}
            onConfirm={(val: any) => {
              console.log(val);
              // const selectedOption = val.selectedOptions[0];
              // console.log(selectedOption, 'selectedOption');
              // forms.schoolId = selectedOption.value;
              // // forms.schoolName = selectedOption.text;
              forms.year = [val.selectedValues[0]];
              forms.yearName = val.selectedValues[0];
              forms.page = 1;
              refreshing.value = true;
              getStat();
              getList();
              forms.yearStatus = false;
            }}
          />
        </Popup>

        <Popup
          v-model:show={forms.perponStatus}
          position="bottom"
          round
          class={'popupBottomSearch'}>
          <Picker
            showToolbar
            columns={forms.sortList}
            onCancel={() => (forms.perponStatus = false)}
            onConfirm={(val: any) => {
              const selectedOption = val.selectedOptions[0];
              if (
                selectedOption.value === 'desc' ||
                selectedOption.value === 'asc'
              ) {
                forms.sortId = selectedOption.value;
                forms.sortKey = 'CLASS';
              }
              if (selectedOption.value === 'mdesc') {
                forms.sortId = 'desc';
                forms.sortKey = 'MEMBER';
              }
              if (selectedOption.value === 'masc') {
                forms.sortId = 'asc';
                forms.sortKey = 'MEMBER';
              }

              forms.sortName = selectedOption.text;
              refreshing.value = true;
              getList();
              forms.perponStatus = false;
            }}
          />
        </Popup>

        <Popup v-model:show={forms.qrcodeStatus} class={styles.popupQrcode}>
          <i
            class={styles.iconClose}
            onClick={() => (forms.qrcodeStatus = false)}></i>

          <img src={imgs.image} class={styles.image} />
          {/*

          <Button
            type="primary"
            round
            block
            class={styles.downloadBtn}
            onClick={downImg}>
            下载二维码
          </Button> */}
          <p class={styles.btnText}>长按或截图保存二维码</p>
        </Popup>

        <div class={styles.templateSection}>
          <div class={[styles.shareContaienr]} id="preview-container">
            <img src={qrcodeBg} class={styles.qrcodeBg} />
            <div class={styles.sectionGroup}>
              <div class={styles.section}>
                <div class={styles.memo}>{forms.urlItem.schoolName}</div>
                <div class={styles.qrcodeSection}>
                  <img src={qrbg} class={styles.qrbg} />
                  <MQrcode
                    class={styles.mqrcode}
                    text={forms.url}
                    logoSize={'small'}
                    size={'100%'}
                  />
                </div>
              </div>
            </div>
          </div>
        </div>
      </div>
    );
  }
});
